package de.bioinf.appl;

import de.bioinf.base.CodonSequence;
import de.bioinf.base.NucleotidSequence;
import de.bioinf.base.NucleotidSequences;
import de.bioinf.utils.BioinfException;
import de.bioinf.utils.Extractor;
import de.bioinf.utils.Logger;
import de.bioinf.utils.Source;
import de.bioinf.utils.Utils;

/**
 * This application is very little, just to extract and print on standard output the last (third) nucleotids of given
 * codon positions.
 * 
 * @author ng
 */
public class NucleotidExtractorAppl extends Appl {
	public static final String APPL_ID = "nucex";

	public static void main(String[] args) {
		try {
			new NucleotidExtractorAppl(args).run();
		} catch (Exception e) {
			Logger.error(e);
		}
	}

	public NucleotidExtractorAppl(String[] args) throws BioinfException {
		super(APPL_ID, args, false);
	}

	@Override
	public void run() throws BioinfException {
		String p_filename = ApplParam.get("-f", Appl.cmdparams, (String) null);
		String p_pos = ApplParam.get("-p", Appl.cmdparams, (String) null);

		BioinfException.fire(p_filename != null && p_pos != null, "Please specify -f and -p!");

		String[] spos = Utils.toArray(p_pos, ",");
		int[] pos = new int[spos.length];
		for (int i = 0; i < pos.length; i++) {
			pos[i] = Integer.parseInt(spos[i]) - 1;
		}

		Extractor extractor = new Extractor("*|#|");

		NucleotidSequences nseqs = NucleotidSequences.loadFasta(new Source(p_filename));
		for (NucleotidSequence nseq : nseqs) {
			System.out.print(extractor.get(nseq.getHeader().toString()).get(0));
			CodonSequence cseq = new CodonSequence(nseq);
			for (int i = 0; i < pos.length; i++) {
				System.out.print(String.format("\t%1s", cseq.getCodon(pos[i]).getNucleotid(2)));
			}
			System.out.println();
		}
	}
}
