package de.bioinf.base;

import java.util.ArrayList;

import de.bioinf.utils.BioinfException;
import de.bioinf.utils.LineReader;
import de.bioinf.utils.Logger;
import de.bioinf.utils.SequenceSelector;
import de.bioinf.utils.Source;

/**
 * A class providing access to Fasta data in files.
 * 
 * @author ng
 */
public class FastaSeqs extends ArrayList<FastaSeq> {

	/**
	 * Loads all sequences of a fasta file. A fasta sequence file must look like {">" header newline
	 * {data newline}}. That means a header is the first (starting with ">") followed by many lines
	 * of data.
	 * 
	 * @param selector - providing the possibility to select only specific sequences.
	 */
	public static FastaSeqs load(Source source, SequenceSelector selector) throws BioinfException {
		LineReader reader = source.getReader(true);
		int lineno = 0;

		FastaSeqs seqs = new FastaSeqs();
		try {
			FastaSeq seq = null;
			while ((seq = FastaSeq.nextSeq(reader, selector)) != null) {
				seqs.add(seq);
			}
		} catch (Exception e) {
			throw BioinfException.convert(e).addMessage(
					String.format("Error in line %d of file '%s'!", lineno, source.getLocation()));
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (Exception e) {
					Logger.error(String.format("Error closing file %s!'", source.getLocation()));
				}
			}
		}

		Logger.info("");
		return seqs;
	}
}
