package de.bioinf.base;

/**
 * A nucleotid is the minimal information in a DNA/RNA sequence. But internal
 * there is no object of this class (up to now). Nucleotids are stored as
 * characters. This class provides some helpful operations on nucleotids.
 * 
 * @author ng
 */
public class Nucleotid {
	public static final char ADENIN = 'A';
	public static final char CYTOSIN = 'C';
	public static final char GUANIN = 'G';
	public static final char THYMIN = 'T';
	public static final char URACIL = 'U';
	/**
	 * The error nucleotid
	 */
	public static final char UNKNOWN = ' ';

	/**
	 * Number of possible nucleotids.
	 */
	public static final int COUNT = 4;
	/**
	 * Mapping char (nucleotid) to int
	 */
	private static int[] nuc2Idx = null;
	/**
	 * Mapping index to nucleotid (DNA)
	 */
	private static char[] idx2nucDNA = null;
	/**
	 * Mapping index to nucleotid (RNA)
	 */
	private static char[] idx2nucRNA = null;

	// Avoid having objects of this type!
	private Nucleotid() {
	}

	public static int getIndex(char nucleotid) {
		return nuc2Idx[nucleotid];
	}

	public static char getDnaNucleotid(int index) {
		return index < idx2nucDNA.length ? idx2nucDNA[index] : UNKNOWN;
	}

	public static char getRnaNucleotid(int index) {
		return index < idx2nucRNA.length ? idx2nucRNA[index] : UNKNOWN;
	}

	/**
	 * It the given nucleotid value a valid value?
	 */
	public static boolean isValid(char nucleotid) {
		return getIndex(nucleotid) != -1;
	}

	/**
	 * It the given nucleotid sequence a valid sequence?
	 */
	public static boolean isValid(String nucleotids) {
		for (int i = 0; i < nucleotids.length(); i++) {
			if (!isValid(nucleotids.charAt(i))) {
				return false;
			}
		}
		return true;
	}

	static {
		nuc2Idx = new int[256];
		for (int i = 0; i < nuc2Idx.length; i++) {
			nuc2Idx[i] = -1;
		}
		nuc2Idx[ADENIN] = 0;
		nuc2Idx[CYTOSIN] = 1;
		nuc2Idx[GUANIN] = 2;
		nuc2Idx[THYMIN] = 3;
		nuc2Idx[URACIL] = 3;

		idx2nucDNA = new char[] { ADENIN, CYTOSIN, GUANIN, THYMIN };
		idx2nucRNA = new char[] { ADENIN, CYTOSIN, GUANIN, URACIL };
	}
}
