package de.bioinf.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import de.bioinf.utils.SelectionEvent;
import de.bioinf.utils.SelectionListener;

/**
 * A list display for a color mapping.
 * 
 * @author ng
 */
public class ColorMapList extends JPanel implements InputComponent {
	private List g_list = null;
	private ColorMap colormap = null;
	private ColorMap savedColormap = null;

	private ArrayList<SelectionListener<ColorMapList>> listeners = new ArrayList<SelectionListener<ColorMapList>>();

	private class ColorWrapper {
		private String key = null;
		private Color color = null;

		private ColorWrapper(String key, Color color) {
			super();
			this.key = key;
			this.color = color;
		}
	}

	private class Model extends DefaultListModel {
		private Model(ColorMap colormap) {
			super();
			for (String key : colormap.getSortedKeys()) {
				addElement(new ColorWrapper(key, colormap.getValue(key)));
			}
		}
	}

	private class List extends JList {
		private class Listener implements ListSelectionListener {
			public void valueChanged(ListSelectionEvent ev) {
				if (!ev.getValueIsAdjusting()) {
					for (SelectionListener<ColorMapList> listener : listeners) {
						listener.selected(new SelectionEvent<ColorMapList>(ColorMapList.this));
					}
				}
			}
		}

		private class Renderer extends DefaultListCellRenderer {
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean selected,
					boolean focussed) {
				ColorWrapper wrapper = (ColorWrapper) value;
				JLabel label = (JLabel) super.getListCellRendererComponent(list, value, index, selected, focussed);
				label.setText(wrapper.key);
				label.setForeground(wrapper.color);
				if (selected) {
					label.setBackground(Color.WHITE);
				}
				return (label);
			}
		}

		public List() {
			setCellRenderer(new Renderer());
			addListSelectionListener(new Listener());
		}
	}

	public ColorMapList(ColorMap colormap) {
		super(new BorderLayout());
		add(new JScrollPane(g_list = new List()), BorderLayout.CENTER);
		init(colormap);
	}

	public void addSelectionListener(SelectionListener<ColorMapList> listener) {
		listeners.add(listener);
	}

	public ColorMap getColorMap() {
		return colormap;
	}

	/**
	 * Returns the selected color or null if nothing is selected
	 */
	public Color getSelectedColor() {
		ColorWrapper wrapper = (ColorWrapper) g_list.getSelectedValue();
		return wrapper != null ? wrapper.color : null;
	}

	/**
	 * Put the color to the actual selected element in the list. Nothing happens if no element is selected.
	 */
	public void setSelectedColor(Color color) {
		ColorWrapper wrapper = (ColorWrapper) g_list.getSelectedValue();
		if (wrapper != null) {
			wrapper.color = color;
			colormap.put(wrapper.key, wrapper.color);
			repaint();
		}
	}

	/**
	 * Initialize the list with the color mapping.
	 */
	public void init(ColorMap colormap) {
		savedColormap = new ColorMap(colormap);
		g_list.setModel(new Model(this.colormap = colormap));
	}

	/**
	 * Refresh the list (e.g. if the color mapping has changed)
	 */
	public void refresh() {
		g_list.setModel(new Model(colormap));
	}

	@Override
	public boolean isOk() {
		return true;
	}

	@Override
	public void reset() {
		if (savedColormap != null) {
			init(savedColormap);
		}
	}

	@Override
	public void saveState() {
		savedColormap = new ColorMap(colormap);
	}
}
