package de.bioinf.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;

import javax.swing.JPanel;

import de.bioinf.utils.SelectionEvent;
import de.bioinf.utils.SelectionListener;

/**
 * A panel displaying a list of key/color entries. Additionally a ColorSelector lets you change the colors of the
 * entries.
 * 
 * @author ng
 */
public class ColorMapPanel extends JPanel implements InputComponent {
	private ColorMapList list = null;
	private ColorSelector selector = null;

	private class ListListener implements SelectionListener<ColorMapList> {
		public void selected(SelectionEvent<ColorMapList> ev) {
			Color color = ev.getComponent().getSelectedColor();
			if (color != null) {
				selector.setSelectedColor(color);
			}
		}
	}

	private class SelectorListener implements SelectionListener<ColorSelector> {
		public void selected(SelectionEvent<ColorSelector> ev) {
			if (list.getSelectedColor() != null) {
				list.setSelectedColor(ev.getComponent().getSelectedColor());
			}
		}
	}

	/**
	 * Create a panel for the colormap
	 */
	public ColorMapPanel(ColorMap colormap) {
		super(new BorderLayout());

		list = new ColorMapList(colormap);
		add(list, BorderLayout.CENTER);

		selector = new ColorSelector();
		selector.setPreferredSize(new Dimension(200, 60));
		add(selector, BorderLayout.SOUTH);

		list.addSelectionListener(new ListListener());
		selector.addSelectionListener(new SelectorListener());
	}

	public ColorMap getColorMap() {
		return list.getColorMap();
	}

	public void init(ColorMap colormap) {
		list.init(colormap);
	}

	/**
	 * Refresh the panel if the colormap changes.
	 */
	public void refresh() {
		list.refresh();
	}

	@Override
	public boolean isOk() {
		return true;
	}

	@Override
	public void reset() {
		list.reset();
	}

	@Override
	public void saveState() {
		list.saveState();
	}
}
