package de.bioinf.ui;

/**
 * Fields that preserve a mechanism to reset an initial state. Usually used as
 * fields in InputDialog.
 * 
 * @author ng
 */
public interface InputComponent {
	/**
	 * Resets the field to the initial state.
	 */
	public void reset();

	/**
	 * Saves the actual state (to be able to do a reset).
	 */
	public void saveState();

	/**
	 * Returns whether the component's value/selection is ok, or not. That means
	 * it is correct for itself, not compared with others. E.g. the entered
	 * characters in an integer field must be digits.
	 */
	public boolean isOk();
}
