package de.bioinf.ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;

public class SimpleColorSelector extends JPanel implements InputComponent, ActionListener {
	private ColorButton selectedButton = null;
	private ColorButton savedButton = null;

	private static String[] DEFAULT_COLORS = new String[] { "0xF0F0F0", "0xF0C0C0", "0xF0C090", "0xF0F090", "0xF0F0C0",
			"0x90F090", "0x90F0F0", "0xC0F0F0", "0xC0C0F0", "0xF0C0F0", "0xC0C0C0", "0xF06060", "0xF09060", "0xF0F060",
			"0xF0F030", "0x60F090", "0x30F0F0", "0x60F0F0", "0x9090F0", "0xF090F0", "0xB0B0B0", "0xF00000", "0xF09000",
			"0xF0C060", "0xF0F000", "0x30F030", "0x60C0C0", "0x30C0F0", "0x6060C0", "0xC060C0", "0x909090", "0xC00000",
			"0xF06000", "0xF0C030", "0xF0C000", "0x30C000", "0x00C0C0", "0x3060F0", "0x6030F0", "0xC030C0", "0x606060",
			"0x900000", "0xC06000", "0xC09030", "0x909000", "0x009000", "0x309090", "0x3030F0", "0x6000C0", "0x903090",
			"0x303030", "0x600000", "0x903000", "0x906030", "0x606000", "0x006000", "0x306060", "0x000090", "0x303090",
			"0x603060", "0x000000", "0x300000", "0x603000", "0x603030", "0x303000", "0x003000", "0x003030", "0x000060",
			"0x300090", "0x300030" };

	class ColorButton extends Button {
		public ColorButton(Color c) {
			super("");
			setBackground(c);
			addActionListener(SimpleColorSelector.this);
		}

		@Override
		public void paint(Graphics g) {
			super.paint(g);
			if (this == selectedButton) {
				g.setColor(Color.WHITE);
				g.fillRect(getWidth() / 2 - 2, getHeight() / 2 - 2, 4, 4);
			}
		}
	}

	/**
	 * Creates a color selector with 70 predefined colors
	 */
	public SimpleColorSelector() {
		super(new GridLayout(7, 10, 2, 2));
		for (String s : DEFAULT_COLORS) {
			add(new ColorButton(new Color(Integer.decode(s).intValue() * 256)));
		}
	}

	/**
	 * Creates a color selector with the given colors
	 */
	public SimpleColorSelector(Color[] colors, int rows, int columns) {
		super(new GridLayout(rows, columns, 2, 2));
		for (Color c : colors) {
			add(new ColorButton(c));
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		selectedButton = (ColorButton) e.getSource();
	}

	/**
	 * Returns the selected color or null if nothing is selected.
	 */
	public Color getSelectedColor() {
		return selectedButton != null ? selectedButton.getBackground() : null;
	}

	/**
	 * Select a color. Note: only the visible colors of the panel are selectable! That means selecting a different color
	 * will result in no action.
	 */
	public void setSelectedColor(Color c) {
		for (int i = 0; i < getComponentCount(); i++) {
			ColorButton b = (ColorButton) getComponent(i);
			if (b.getBackground().equals(c)) {
				selectedButton = b;
			}
		}
	}

	public boolean isOk() {
		return true;
	}

	public void reset() {
		selectedButton = savedButton;
	}

	public void saveState() {
		savedButton = selectedButton;
	}
}
