package de.bioinf.ui.graph;

import java.awt.Graphics;

import de.bioinf.utils.ValueRange;

/**
 * The base class for elements to be added to a coordinate system's panel.
 * 
 * @author ng
 */
public abstract class CoordElem extends GraphElem {
	/**
	 * The base panel (upon which this element will be displayed). That panel is used to transform x/y values of the
	 * element to the corresponding x/y values of the display.
	 */
	private CoordPanel panel = null;
	/**
	 * The original x value
	 */
	private double xval = 0;
	/**
	 * The original y value
	 */
	private double yval = 0;

	/**
	 * Creates a coordinate element, but does NOT add it to the underlaying panel.
	 * 
	 * @param xval - the x axis value (NOT the position of the display)
	 * @param yval - the y axis value (NOT the position of the display)
	 * @param id - information that are displayed when clicked.
	 */
	public CoordElem(CoordPanel panel, double xval, double yval, String id) {
		super(id);
		this.panel = panel;
		this.xval = xval;
		this.yval = yval;
	}

	@Override
	public void adjustRanges(ValueRange xrange, ValueRange yrange) {
		xrange.add(xval);
		yrange.add(yval);
	}

	@Override
	public boolean isHit(int x, int y) {
		return false;
	}

	@Override
	public boolean isVisible() {
		return true;
	}

	@Override
	public void paint(Graphics g) {
	}

	/**
	 * Returns original value that must be transformed to the x axis
	 */
	public final double getXVal() {
		return xval;
	}

	/**
	 * Returns original value that must be transformed to the y axis
	 */
	public final double getYVal() {
		return yval;
	}

	/**
	 * Returns the x position
	 */
	public final int getX() {
		return panel.getX(this);
	}

	/**
	 * Returns the y position
	 */
	public final int getY() {
		return panel.getY(this);
	}

	protected CoordPanel getPanel() {
		return panel;
	}
}
