package de.bioinf.utils;

/**
 * A 1-dimensional integer value range.
 * 
 * @author ng
 */
public class IntValueRange {
	private int min = Integer.MAX_VALUE;
	private int max = Integer.MIN_VALUE;

	public IntValueRange() {
		super();
	}

	public IntValueRange(int min, int max) {
		super();
		set(min, max);
	}

	/**
	 * "Add" a value and calculate the limits anew.
	 */
	public void add(int value) {
		min = Math.min(min, value);
		max = Math.max(max, value);
	}

	/**
	 * "Add" a range and calculate the limits anew.
	 */
	public void add(IntValueRange range) {
		if (range.isValid()) {
			min = Math.min(min, range.min);
			max = Math.max(max, range.max);
		}
	}

	public boolean equals(Object o) {
		if (o == null || !(o instanceof IntValueRange)) {
			return false;
		}
		IntValueRange r = (IntValueRange) o;
		return this.min == r.min && this.max == r.max;
	}

	/**
	 * Returns the minimum of all added values.
	 * <p>
	 * Remark: If nothing is added Integer.MAX_VALUE will be returned!
	 */
	public int getMinVal() {
		return min;
	}

	/**
	 * Returns the maximum of all added values.
	 * <p>
	 * Remark: If nothing is added Integer.MIN_VALUE will be returned!
	 */
	public int getMaxVal() {
		return max;
	}

	/**
	 * Returns the difference between the maximum (added) value and the minimum (added) value.
	 */
	public int getRange() {
		return max - min;
	}

	public void init() {
		min = Integer.MAX_VALUE;
		max = Integer.MIN_VALUE;
	}

	/**
	 * Do the 2 ranges intersect?
	 */
	public boolean intersects(IntValueRange range) {
		return isInside(range.min) || range.isInside(min);
	}

	/**
	 * Is the value inside the range (including borders (min/max))?
	 */
	public boolean isInside(int value) {
		return min <= value && value <= max;
	}

	/**
	 * Is this a valid range (max >= min)?
	 */
	public boolean isValid() {
		return max >= min;
	}

	/**
	 * Set the limts.
	 */
	public void set(int min, int max) {
		this.min = min;
		this.max = max;
	}

	public String toString() {
		return "[" + min + ", " + max + "]";
	}
}
