package de.bioinf.utils;

import java.util.Date;
import java.util.Properties;

import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

/**
 * A class for sending a mail.
 * 
 * <p>
 * To do mailing, the jar archives: activation.jar, dns.jar, imap.jar,
 * mailapi.jar, pop3.jar and smtp.jar should be in the build path. (See projext
 * in the internet: jaf-1.0.2 and javamail-1.4.1)
 * 
 * @author ng
 */
public class Mail {
	static class MailAuthenticator extends Authenticator {
		private final String user;
		private final String password;

		public MailAuthenticator(String user, String password) {
			this.user = user;
			this.password = password;
		}

		protected PasswordAuthentication getPasswordAuthentication() {
			return new PasswordAuthentication(this.user, this.password);
		}
	}

	/**
	 * Send a mail
	 * 
	 * @param smtphost - Like mail.uni-meunster.de
	 * @param username - The central name / "Nutzerkennung"
	 * @param password - The central password
	 * @param sender - Like username@domain.de
	 * @param receiver - Like username@domain.de
	 * @param subject - The header line
	 * @param text - The text
	 */
	public static void send(String smtphost, String username, String password, String sender, String receiver,
			String subject, String text) throws BioinfException {
		MailAuthenticator auth = new MailAuthenticator(username, password);
		Properties props = new Properties();
		props.put("mail.smtp.host", smtphost);
		props.put("mail.smtp.auth", "true");
		Session session = Session.getDefaultInstance(props, auth);
		try {
			Message msg = new MimeMessage(session);
			msg.setFrom(new InternetAddress(sender));
			msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(receiver, false));
			msg.setSubject(subject);
			msg.setText(text);
			// msg.setHeader("Test", "Test");
			msg.setSentDate(new Date());
			Transport.send(msg);
		} catch (Exception e) {
			throw BioinfException.convert(e);
		}
	}
}