package de.bioinf.utils;

import java.util.StringTokenizer;

/**
 * A simple expression matcher. The expression may contain special characters like "*", "." or "\"
 * to match a set of characters (even empty!), one character and to exclude a character from their
 * special meaning, that means to treat it as it is.
 * 
 * @author ng
 */
public class Matcher {
	private String pattern = null;

	public Matcher(String pattern) {
		super();
		this.pattern = pattern;
	}

	public boolean matches(String str) {
		int pos = 0;
		boolean escaped = false;
		boolean multi = false;
		StringTokenizer tok = new StringTokenizer(pattern, "*.\\", true);
		while (tok.hasMoreTokens()) {
			String token = tok.nextToken();

			if (!escaped) {
				if ("*".equals(token)) {
					multi = true;
					continue;
				} else if (".".equals(token)) {
					pos++;
					continue;
				} else if ("\\".equals(token)) {
					escaped = true;
					continue;
				}
			} else {
				escaped = false;
			}

			if (multi) {
				pos = str.indexOf(token, pos);
				multi = false;
			} else {
				int newpos = str.indexOf(token, pos);
				if (newpos != pos) {
					return false;
				}
				pos = newpos;
			}

			if (pos == -1 || (!tok.hasMoreTokens() && str.length() - pos > token.length())) {
				return false;
			}

			pos += token.length();
		}
		return true;
	}

	public String toString() {
		return pattern;
	}
}
